/**
  ******************************************************************************
  * @file    driver_iic_pan2025.c
  * @author  caixiaohai@panchip.com
  * @brief   I2C HAL module driver.
  *          This file provides firmware functions to manage the following
  *          functionalities of the Inter Integrated Circuit (I2C) peripheral:
  *           + Initialization and de-initialization functions
  *           + IO operation functions
  *           + Peripheral State, Mode and Error functions
  *
  @verbatim
  ==============================================================================
                        ##### How to use this driver #####
  ==============================================================================
  **/

#include "driver_iic.h"
#include "driver_delay.h"

/**
  * @brief I2C MSP Init.
  * @param  hi2c pointer to a I2C_HandleTypeDef structure that contains
  *         the configuration information for I2C module
  * @retval None
  */
 __weak void HAL_I2C_MspInit(I2C_HandleTypeDef *hi2c)
{
  /* Prevent unused argument(s) compilation warning */
  UNUSED(hi2c);
  /* NOTE : This function Should not be modified, when the callback is needed,
            the HAL_I2C_MspInit could be implemented in the user file
   */
}

/**
  * @brief I2C MSP DeInit
  * @param  hi2c pointer to a I2C_HandleTypeDef structure that contains
  *         the configuration information for I2C module
  * @retval None
  */
 __weak void HAL_I2C_MspDeInit(I2C_HandleTypeDef *hi2c)
{
  /* Prevent unused argument(s) compilation warning */
  UNUSED(hi2c);
  /* NOTE : This function Should not be modified, when the callback is needed,
            the HAL_I2C_MspDeInit could be implemented in the user file
   */
}

/**
  * @brief  Initializes the I2C according to the specified parameters
  *         in the I2C_InitTypeDef and create the associated handle.
  * @param  hi2c pointer to a I2C_HandleTypeDef structure that contains
  *         the configuration information for I2C module
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_init(I2C_HandleTypeDef *hi2c)
{
    uint32_t u32Div;
    if(hi2c->Instance==NULL)
    {
        return HAL_ERROR;
    }

    if(hi2c->State == HAL_I2C_STATE_RESET)
    {
        /* Allocate lock resource and initialize it */
        hi2c->Lock = HAL_UNLOCKED;
        /* Init the low level hardware : GPIO, CLOCK, NVIC */
        HAL_I2C_MspInit(hi2c);
    }

    hi2c->State = HAL_I2C_STATE_BUSY;

    /* Disable the selected I2C peripheral */
    __HAL_I2C_DISABLE(hi2c);

    /* Compute proper divider for I2C clock */
    u32Div = (uint32_t) ((SystemCoreClock+hi2c->ClockSpeed)/(hi2c->ClockSpeed *4) - 1); 
    if(u32Div>0xff)   u32Div = 0xff;         
    hi2c->Instance->CLKDIV = u32Div;

    /* Enable the selected I2C peripheral */
    __HAL_I2C_ENABLE(hi2c);

    hi2c->ErrorCode = HAL_I2C_ERROR_NONE;
    hi2c->State = HAL_I2C_STATE_READY;
    hi2c->PreviousState = I2C_STATE_NONE;
    hi2c->Mode = HAL_I2C_MODE_NONE; 

    return HAL_OK;
}
/**
  * @brief  DeInitializes the I2C peripheral.
  * @param  hi2c pointer to a I2C_HandleTypeDef structure that contains
  *         the configuration information for I2C module
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_DeInit(I2C_HandleTypeDef *hi2c)
{
    /* Check the I2C handle allocation */
    if(hi2c == NULL)
    {
        return HAL_ERROR;
    }

    hi2c->State = HAL_I2C_STATE_BUSY; 

    /* Disable the I2C Peripheral Clock */
    __HAL_I2C_DISABLE(hi2c);

    /* DeInit the low level hardware: GPIO, CLOCK, NVIC */
    HAL_I2C_MspDeInit(hi2c);

    hi2c->ErrorCode     = HAL_I2C_ERROR_NONE;
    hi2c->State         = HAL_I2C_STATE_RESET;
    hi2c->PreviousState = I2C_STATE_NONE;
    hi2c->Mode          = HAL_I2C_MODE_NONE;

    /* Release Lock */
    __HAL_UNLOCK(hi2c);

    return HAL_OK;
}

/**
  * @brief  This function handles I2C Communication Timeout for I2C status.
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *         the configuration information for I2C module
  * @param  Status specifies the I2C Status to check.
  * @param  Timeout Timeout duration
  * @param  Tickstart Tick start value
  * @retval HAL status
  */
static HAL_StatusTypeDef I2C_WaitStatusUntilTimeout(I2C_HandleTypeDef *hi2c, uint32_t Status, uint32_t Timeout, uint32_t Tickstart)
{
	//while((!((hi2c->Instance)->CTL & I2C_CTL_SI_Msk))||(I2C_GET_STATUS(hi2c->Instance)!=Status))
	while(I2C_GET_STATUS(hi2c->Instance)!=Status)
	{
    /* Check for the Timeout */
    if(Timeout != HAL_MAX_DELAY)
    {
      if((Timeout == 0U)||((HAL_GetTick() - Tickstart ) > Timeout))
      {
        hi2c->PreviousState = I2C_STATE_NONE;
        hi2c->State= HAL_I2C_STATE_READY;

        /* Process Unlocked */
        __HAL_UNLOCK(hi2c);

        return HAL_TIMEOUT;
      }
    }
  }
  return HAL_OK;
}

/**
  * @brief  Write an amount of data in blocking mode to a specific memory address
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @param  DevAddress Target device address
  * @param  MemAddress Internal memory address
  * @param  MemAddSize Size of internal memory address
  * @param  pData Pointer to data buffer
  * @param  Size Amount of data to be sent
  * @param  Timeout Timeout duration
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_Mem_Write(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress , uint8_t *pData, uint16_t Size, uint32_t Timeout)
{
  uint32_t tickstart = 0x00U;
  /* Init tickstart for timeout management*/
  tickstart = HAL_GetTick();
  if(hi2c->State == HAL_I2C_STATE_READY)
  {
    /* Process Locked */
    __HAL_LOCK(hi2c);

    hi2c->State     = HAL_I2C_STATE_BUSY_TX;
    hi2c->Mode      = HAL_I2C_MODE_MEM;
    hi2c->ErrorCode = HAL_I2C_ERROR_NONE;

    /* Prepare transfer parameters */
    hi2c->pBuffPtr    = pData;
    hi2c->XferCount   = 0;
    hi2c->XferSize    = Size;

    /* Process unLocked */
    __HAL_UNLOCK(hi2c);

    /*##-1- Send Start signal ################################*/
    I2C_SET_CONTROL_REG(hi2c->Instance,I2C_SI|I2C_STA); 
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STAUTS_START,Timeout,tickstart)!=HAL_OK)
    {
      return HAL_ERROR;
    }

    /*##-2- Send Slave Address ################################*/
    I2C_SET_DATA(hi2c->Instance, DevAddress<<1);
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_TRANSMIT_ADR_ACK,Timeout,tickstart)!=HAL_OK)
    {
      if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_TRANSMIT_ADR_NACK)
      {
        return HAL_ERROR;
      }
      else
      {
        return HAL_TIMEOUT;
      }
    }

    /*##-3- Send Memory Address ################################*/
    I2C_SET_DATA(hi2c->Instance, MemAddress & 0xFFUL); 
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_TRANSMIT_DAT_ACK,Timeout,tickstart)!=HAL_OK)
    {
      if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_TRANSMIT_DAT_NCAK)
      {
        return HAL_ERROR;
      }
      else
      {
        return HAL_TIMEOUT;
      }
		}

    /*##-4- Write data ######################################*/
    while(hi2c->XferCount<hi2c->XferSize)
    {
				I2C_SET_DATA(hi2c->Instance, hi2c->pBuffPtr[hi2c->XferCount++]); 
				I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
        if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_TRANSMIT_DAT_ACK,Timeout,tickstart)!=HAL_OK)
        {
          if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_TRANSMIT_DAT_NCAK)
          {
            return HAL_ERROR;
          }
          else
          {
            return HAL_TIMEOUT;
          }
        }
    }

    /*##-5- Send Stop ######################################*/
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_BUS_RELEASED)
    {
      return HAL_ERROR;
    }

    /* Process Unlocked */
    hi2c->State     = HAL_I2C_STATE_READY;
    __HAL_UNLOCK(hi2c);
    return HAL_OK;
  }
  else
  {
    return HAL_BUSY;
  }
}

/**
  * @brief  Read an amount of data in blocking mode from a specific memory address
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @param  DevAddress Target device address
  * @param  MemAddress Internal memory address
  * @param  pData Pointer to data buffer
  * @param  Size Amount of data to be sent
  * @param  Timeout Timeout duration
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_Mem_Read(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size, uint32_t Timeout)
{
  uint32_t tickstart = 0x00U;

  /* Init tickstart for timeout management*/
  tickstart = HAL_GetTick();
  if(hi2c->State == HAL_I2C_STATE_READY)
  {
    /* Wait until BUSY flag is reset */
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_BUS_RELEASED,Timeout,tickstart)!=HAL_OK)
    {
      return HAL_BUSY;
    }

    /* Process locked */
    __HAL_LOCK(hi2c);

    hi2c->State     = HAL_I2C_STATE_BUSY_RX;
    hi2c->Mode      = HAL_I2C_MODE_MEM;
    hi2c->ErrorCode = HAL_I2C_ERROR_NONE;

    /* Prepare transfer parameters */
    hi2c->pBuffPtr    = pData;
    hi2c->XferCount   = 0;
    hi2c->XferSize    = Size;

    /* Process Unlocked */
    __HAL_UNLOCK(hi2c);


    /*##-1- Send Start signal ################################*/
    I2C_SET_CONTROL_REG(hi2c->Instance,  I2C_SI|I2C_STA);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STAUTS_START,Timeout,tickstart)!=HAL_OK)
    {
      return HAL_ERROR;
    }

    /*##-2- Send Slave Address with 'W'################################*/
    I2C_SET_DATA(hi2c->Instance, DevAddress<<1);
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_TRANSMIT_ADR_ACK,Timeout,tickstart)!=HAL_OK)
    {
      if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_TRANSMIT_ADR_NACK)
      {
        return HAL_ERROR;
      }
      else
      {
        return HAL_TIMEOUT;
      }
    }

    /*##-3- Send Memory Address ################################*/
    I2C_SET_DATA(hi2c->Instance, MemAddress & 0xFFUL); 
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_TRANSMIT_DAT_ACK,Timeout,tickstart)!=HAL_OK)
    {
      if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_TRANSMIT_DAT_NCAK)
      {
        return HAL_ERROR;
      }
      else
      {
        return HAL_TIMEOUT;
      }
		}

    /*##-4- ReStart #########################################*/
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STA | I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_RESTATRT,Timeout,tickstart)!=HAL_OK)
    {
      return HAL_ERROR;
    }

    /*##-5- Send Slave Address with 'R'################################*/
    I2C_SET_DATA(hi2c->Instance, (DevAddress<<1)|0x01); 
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_RECEIVE_ADR_ACK,Timeout,tickstart)!=HAL_OK)
    {
      if(I2C_GET_STATUS(hi2c->Instance)==I2C_M_STATUS_RECEIVE_ADR_NACK)
      {
        return HAL_ERROR;
      }
      else
      {
        return HAL_TIMEOUT;
      }
    }

    /*##-6- Read data sequence ######################################*/
    while(hi2c->XferCount<(hi2c->XferSize-1))
    {
      I2C_SET_CONTROL_REG(hi2c->Instance, I2C_AA|I2C_SI);
      if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_RECEIVE_DAT_ACK,Timeout,tickstart)!=HAL_OK)
      {
        return HAL_ERROR;  
      }
      hi2c->pBuffPtr[hi2c->XferCount++] = I2C_GET_DATA(hi2c->Instance);
    }
		
		/*##-7- Read last data ######################################*/
		I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_RECEIVE_DAT_NACK,Timeout,tickstart)!=HAL_OK)
    {
      return HAL_ERROR;  
    }
    hi2c->pBuffPtr[hi2c->XferCount] = I2C_GET_DATA(hi2c->Instance);

    /*##-8- Send Stop ######################################*/
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    if(I2C_WaitStatusUntilTimeout(hi2c,I2C_M_STATUS_BUS_RELEASED,Timeout,tickstart)!=HAL_OK)
    {
       return HAL_ERROR;
    }

    hi2c->State = HAL_I2C_STATE_READY;
    return HAL_OK;
  }
  else
  {
    return HAL_BUSY;
  }
  
}

/**
  * @brief  This function handles I2C TX interrupt request.
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @retval None
  */
void HAL_I2C_Tx_IRQHandler(I2C_HandleTypeDef *hi2c)
{
  if(hi2c->Instance->STATUS == I2C_M_STAUTS_START)                    /* Strat-Signal has been transmitted  */
  {
    I2C_SET_DATA(hi2c->Instance, hi2c->Devaddress << 1); 
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_ADR_ACK)    /* Slave Address has sent and acknowledge */
  {
    I2C_SET_DATA(hi2c->Instance,   hi2c->Memaddress);
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_ADR_NACK)   /*  Slave Address has sent and not acknowledeg */
  {
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    hi2c->State=HAL_I2C_STATE_READY;
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_DAT_ACK)    /* Memaddress/Data has send and acknowledge */
  {
    
    if(hi2c->XferCount<hi2c->XferSize)
    {
      I2C_SET_DATA(hi2c->Instance, hi2c->pBuffPtr[hi2c->XferCount++]);
      I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
    }
    else
    {
      I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
      hi2c->State=HAL_I2C_STATE_READY;
    }
    
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_DAT_NCAK)   /* Data has send and not acknowledeg */
  {
      I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
      hi2c->State=HAL_I2C_STATE_READY;
  }
  else
  {
    /* code */
  }
  


}

/**
  * @brief  This function handles I2C TX interrupt request.
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @retval None
  */
void HAL_I2C_Rx_IRQHandler(I2C_HandleTypeDef *hi2c)
{
  if(hi2c->Instance->STATUS == I2C_M_STAUTS_START)                    /* Strat-Signal has been transmitted  */
  {
    I2C_SET_DATA(I2C0, hi2c->Devaddress << 1);     
    I2C_SET_CONTROL_REG(I2C0, I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_ADR_ACK)    /* Slave Address with 'W'has sent and acknowledge */
  {
    I2C_SET_DATA(I2C0, hi2c->Memaddress);
    I2C_SET_CONTROL_REG(I2C0, I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_ADR_NACK)   /*  Slave Address with 'W' has sent and not acknowledeg */
  {
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    hi2c->State=HAL_I2C_STATE_READY;
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_DAT_ACK)    /*  memaddress has sent and acknowledeg */
  {
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STA | I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_TRANSMIT_DAT_NCAK)   /*  memaddress has sent and not acknowledeg */
  {
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    hi2c->State=HAL_I2C_STATE_READY;
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_RESTATRT)            /*  ReStart-Signal has been transmitted*/
  {
    I2C_SET_DATA(hi2c->Instance, (hi2c->Devaddress<<1)|0x01); 
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_SI);
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_RECEIVE_ADR_ACK)     /*  Slave Address with 'R'has sent and acknowledeg */
  {
	if(hi2c->XferSize==1)
	{
    I2C_SET_CONTROL_REG(I2C0, I2C_SI);
	}else
	{
		I2C_SET_CONTROL_REG(I2C0, I2C_SI|I2C_AA);
	}
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_RECEIVE_ADR_NACK)    /*  Slave Address with 'R'has sent and not acknowledeg */
  {
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    hi2c->State=HAL_I2C_STATE_READY;
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_RECEIVE_DAT_ACK)
  {
    hi2c->pBuffPtr[hi2c->XferCount++] =I2C_GET_DATA(hi2c->Instance);
    if(hi2c->XferCount<hi2c->XferSize-1)
    {
      I2C_SET_CONTROL_REG(I2C0, I2C_SI|I2C_AA);
    }
    else
    {
      I2C_SET_CONTROL_REG(I2C0, I2C_SI);
      //hi2c->State=HAL_I2C_STATE_READY;
    }
    
  }
  else if(hi2c->Instance->STATUS == I2C_M_STATUS_RECEIVE_DAT_NACK)
  {
	 hi2c->pBuffPtr[hi2c->XferCount] =I2C_GET_DATA(hi2c->Instance);
    I2C_SET_CONTROL_REG(hi2c->Instance, I2C_STO | I2C_SI);
    hi2c->State=HAL_I2C_STATE_READY;
  }
  else
  {
    /* code */
  }
  
}

/**
  * @brief  This function handles I2C  interrupt request.
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @retval None
  */
void HAL_I2C_IRQHandler(I2C_HandleTypeDef *hi2c)
{
  if(hi2c->State==HAL_I2C_STATE_BUSY_TX)
  {
    HAL_I2C_Tx_IRQHandler(hi2c);
  }
  else if(hi2c->State==HAL_I2C_STATE_BUSY_RX)
  {
    HAL_I2C_Rx_IRQHandler(hi2c);
  }
  
}

/**
  * @brief  Write an amount of data in non-blocking mode with Interrupt to a specific memory address
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @param  DevAddress Target device address
  * @param  MemAddress Internal memory address
  * @param  pData Pointer to data buffer
  * @param  Size Amount of data to be sent
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_Mem_Write_IT(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size)
{
  if(hi2c->State == HAL_I2C_STATE_READY)
  {
    /* Process Locked */
    __HAL_LOCK(hi2c);

    hi2c->State = HAL_I2C_STATE_BUSY_TX;
    hi2c->Mode = HAL_I2C_MODE_MEM;
    hi2c->ErrorCode = HAL_I2C_ERROR_NONE;

    /* Prepare transfer parameters */
    hi2c->pBuffPtr = pData;
    hi2c->XferSize = Size;
    hi2c->XferCount = 0;
    hi2c->Devaddress = DevAddress;
    hi2c->Memaddress = MemAddress;
    hi2c->EventCount = 0U;

		/* Enable I2C interrupt */
    __HAL_I2C_ENABLE_IT(hi2c);
    NVIC_EnableIRQ(hi2c->i2cIrq);	

    /* I2C as master sends START signal */
		I2C_SET_CONTROL_REG(I2C0,  I2C_SI|I2C_STA); 

    /* Process Unlocked */
    __HAL_UNLOCK(hi2c);

    return HAL_OK;
  }
  else
  {
    return HAL_BUSY;
  }
  
}

/**
  * @brief  Read an amount of data in non-blocking mode with Interrupt from a specific memory address
  * @param  hi2c Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  * @param  DevAddress Target device address
  * @param  MemAddress Internal memory address
  * @param  pData Pointer to data buffer
  * @param  Size Amount of data to be sent
  * @retval HAL status
  */
HAL_StatusTypeDef i2c_pan2025_Mem_Read_IT(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size)
{
	if(hi2c->State == HAL_I2C_STATE_READY)
	{
    /* Process Locked */
    __HAL_LOCK(hi2c);

    hi2c->State = HAL_I2C_STATE_BUSY_RX;
    hi2c->Mode = HAL_I2C_MODE_MEM;
    hi2c->ErrorCode = HAL_I2C_ERROR_NONE;

    /* Prepare transfer parameters */
    hi2c->pBuffPtr = pData;
    hi2c->XferSize = Size;
    hi2c->XferCount = 0;
    hi2c->Devaddress = DevAddress;
    hi2c->Memaddress = MemAddress;
    hi2c->EventCount = 0U;

		/* Enable I2C interrupt */
    __HAL_I2C_ENABLE_IT(hi2c);
    NVIC_EnableIRQ(hi2c->i2cIrq);	

    /* I2C as master sends START signal */
		I2C_SET_CONTROL_REG(I2C0,  I2C_SI|I2C_STA); 

    /* Process Unlocked */
    __HAL_UNLOCK(hi2c);

    return HAL_OK;
  }
  else
  {
    return HAL_BUSY;
  }
  
}
