/*******************************************************************************
 * @note      Copyright (C) 2017 Shanghai Panchip Microelectronics Co., Ltd.
 *            All rights reserved.
 *
 * @file      driver_iic.h
 * @brief     
 *
 * @history - V1.0, 2019-11-25, caixiaohai@panchip.com, first implementation.
*******************************************************************************/
#ifndef __DRIVER_IIC_H_
#define __DRIVER_IIC_H_

#ifdef __cplusplus
extern "C"{
#endif

#include "pan2025xx.h"
#include "i2c.h"
	
#define UNUSED(X) (void)X      /* To avoid gcc/g++ warnings */

#define HAL_MAX_DELAY      0xFFFFFFFFU

/** 
  * @brief  HAL Status structures definition  
  */ 
typedef enum 
{
  HAL_OK       = 0x00U,
  HAL_ERROR    = 0x01U,
  HAL_BUSY     = 0x02U,
  HAL_TIMEOUT  = 0x03U
} HAL_StatusTypeDef;

/** 
  * @brief  HAL Lock structures definition  
  */
typedef enum 
{
  HAL_UNLOCKED = 0x00U,
  HAL_LOCKED   = 0x01U  
} HAL_LockTypeDef;


#define __HAL_LOCK(__HANDLE__)                                           \
                            do{                                        \
                                if((__HANDLE__)->Lock == HAL_LOCKED)   \
                                {                                      \
                                    return HAL_BUSY;                    \
                                }                                      \
                                else                                   \
                                {                                      \
                                    (__HANDLE__)->Lock = HAL_LOCKED;    \
                                }                                      \
                                }while (0U)

#define __HAL_UNLOCK(__HANDLE__)                                          \
                                do{                                       \
                                    (__HANDLE__)->Lock = HAL_UNLOCKED;    \
                                }while (0U)

/**
  * @brief  HAL Mode structure definition
  * @note  HAL I2C Mode value coding follow below described bitmap :
  *          b7     (not used)
  *             x  : Should be set to 0
  *          b6     (not used)
  *             x  : Should be set to 0
  *          b5
  *             0  : None
  *             1  : Slave (HAL I2C communication is in Slave Mode)
  *          b4
  *             0  : None
  *             1  : Master (HAL I2C communication is in Master Mode)
  *          b3-b2-b1-b0  (not used)
  *             xxxx : Should be set to 0000
  */
typedef enum
{
  HAL_I2C_MODE_NONE               = 0x00U,   /*!< No I2C communication on going             */
  HAL_I2C_MODE_MASTER             = 0x10U,   /*!< I2C communication is in Master Mode       */
  HAL_I2C_MODE_SLAVE              = 0x20U,   /*!< I2C communication is in Slave Mode        */
  HAL_I2C_MODE_MEM                = 0x40U    /*!< I2C communication is in Memory Mode       */
}HAL_I2C_ModeTypeDef;

/**
  * @brief  HAL State structure definition
  * @note  HAL I2C State value coding follow below described bitmap :
  *          b7-b6  Error information 
  *             00 : No Error
  *             01 : Abort (Abort user request on going)
  *             10 : Timeout
  *             11 : Error
  *          b5     IP initilisation status
  *             0  : Reset (IP not initialized)
  *             1  : Init done (IP initialized and ready to use. HAL I2C Init function called)
  *          b4     (not used)
  *             x  : Should be set to 0
  *          b3
  *             0  : Ready or Busy (No Listen mode ongoing)
  *             1  : Listen (IP in Address Listen Mode)
  *          b2     Intrinsic process state
  *             0  : Ready
  *             1  : Busy (IP busy with some configuration or internal operations)
  *          b1     Rx state
  *             0  : Ready (no Rx operation ongoing)
  *             1  : Busy (Rx operation ongoing)
  *          b0     Tx state
  *             0  : Ready (no Tx operation ongoing)
  *             1  : Busy (Tx operation ongoing)
  */
typedef enum
{
  HAL_I2C_STATE_RESET             = 0x00U,   /*!< Peripheral is not yet Initialized         */
  HAL_I2C_STATE_READY             = 0x20U,   /*!< Peripheral Initialized and ready for use  */
  HAL_I2C_STATE_BUSY              = 0x24U,   /*!< An internal process is ongoing            */
  HAL_I2C_STATE_BUSY_TX           = 0x21U,   /*!< Data Transmission process is ongoing      */
  HAL_I2C_STATE_BUSY_RX           = 0x22U,   /*!< Data Reception process is ongoing         */
  HAL_I2C_STATE_LISTEN            = 0x28U,   /*!< Address Listen Mode is ongoing            */
  HAL_I2C_STATE_BUSY_TX_LISTEN    = 0x29U,   /*!< Address Listen Mode and Data Transmission
                                                 process is ongoing                         */
  HAL_I2C_STATE_BUSY_RX_LISTEN    = 0x2AU,   /*!< Address Listen Mode and Data Reception
                                                 process is ongoing                         */
  HAL_I2C_STATE_ABORT             = 0x60U,   /*!< Abort user request ongoing                */
  HAL_I2C_STATE_TIMEOUT           = 0xA0U,   /*!< Timeout state                             */
  HAL_I2C_STATE_ERROR             = 0xE0U    /*!< Error                                     */

}HAL_I2C_StateTypeDef;

/**
  * @brief  I2C handle Structure definition
  */
typedef struct
{
  I2C_T                       *Instance;      /*!< I2C registers base address               */

  uint32_t                    ClockSpeed;                       /*!< Specifies the clock frequency.This parameter must be set to a value lower than 400kHz */   

  IRQn_Type                   i2cIrq; 
                                             
  uint8_t                     *pBuffPtr;      /*!< Pointer to I2C transfer buffer           */
                                             
  uint16_t                    XferSize;       /*!< I2C transfer size                        */
                                             
  __IO uint16_t               XferCount;      /*!< I2C transfer counter                     */
                                             
//  __IO uint32_t              XferOptions;    /*!< I2C transfer options                     */
                                             
  __IO uint32_t               PreviousState;  /*!< I2C communication Previous state and mode
                                              context for internal usage               */

  HAL_LockTypeDef             Lock;           /*!< I2C locking object                       */
                                             
  __IO HAL_I2C_StateTypeDef        State;          /*!< I2C communication state                  */
                                             
  HAL_I2C_ModeTypeDef         Mode;           /*!< I2C communication mode                   */
                                       
  uint32_t                    ErrorCode;      /*!< I2C Error code                           */

  uint32_t                    Devaddress;     /*!< I2C Target device address                */

  __IO uint32_t              Memaddress;      /*!< I2C Target memory address                */

  uint32_t                    EventCount;     /*!< I2C Event counter                        */
	
}I2C_HandleTypeDef;


/** @defgroup I2C_Error_Code I2C Error Code
  * @brief    I2C Error Code 
  * @{
  */ 
#define HAL_I2C_ERROR_NONE       0x00000000U    /*!< No error           */
#define HAL_I2C_ERROR_BERR       0x00000001U    /*!< BERR error         */
#define HAL_I2C_ERROR_ARLO       0x00000002U    /*!< ARLO error         */
#define HAL_I2C_ERROR_AF         0x00000004U    /*!< AF error           */
#define HAL_I2C_ERROR_TIMEOUT    0x00000020U    /*!< Timeout Error      */
/**
  * @}
  */

/** @addtogroup I2C_Private_Define
  * @{
  */  

/* Private define for @ref PreviousState usage */
#define I2C_STATE_MSK             ((uint32_t)((HAL_I2C_STATE_BUSY_TX | HAL_I2C_STATE_BUSY_RX) & (~(uint32_t)HAL_I2C_STATE_READY))) /*!< Mask State define, keep only RX and TX bits            */
#define I2C_STATE_NONE            ((uint32_t)(HAL_I2C_MODE_NONE))                                                        /*!< Default Value                                          */
#define I2C_STATE_MASTER_BUSY_TX  ((uint32_t)((HAL_I2C_STATE_BUSY_TX & I2C_STATE_MSK) | HAL_I2C_MODE_MASTER))            /*!< Master Busy TX, combinaison of State LSB and Mode enum */
#define I2C_STATE_MASTER_BUSY_RX  ((uint32_t)((HAL_I2C_STATE_BUSY_RX & I2C_STATE_MSK) | HAL_I2C_MODE_MASTER))            /*!< Master Busy RX, combinaison of State LSB and Mode enum */
#define I2C_STATE_SLAVE_BUSY_TX   ((uint32_t)((HAL_I2C_STATE_BUSY_TX & I2C_STATE_MSK) | HAL_I2C_MODE_SLAVE))             /*!< Slave Busy TX, combinaison of State LSB and Mode enum  */
#define I2C_STATE_SLAVE_BUSY_RX   ((uint32_t)((HAL_I2C_STATE_BUSY_RX & I2C_STATE_MSK) | HAL_I2C_MODE_SLAVE))             /*!< Slave Busy RX, combinaison of State LSB and Mode enum  */

#define __HAL_I2C_ENABLE(__HANDLE__)                              ((__HANDLE__)->Instance->CTL |=  I2C_CTL_I2CEN_Msk)
#define __HAL_I2C_DISABLE(__HANDLE__)                             ((__HANDLE__)->Instance->CTL &=  ~I2C_CTL_I2CEN_Msk)
#define __HAL_I2C_ENABLE_IT(__HANDLE__)                           ((__HANDLE__)->Instance->CTL |=  I2C_CTL_INTEN_Msk)
#define __HAL_I2C_DISABLE_IT(__HANDLE__)                          ((__HANDLE__)->Instance->CTL &=  ~I2C_CTL_INTEN_Msk)

/**
  * @}
  */

/** @addtogroup I2C Master Mode :Status Code
  * @{
  */ 
#define I2C_M_STAUTS_START                          0x08
#define I2C_M_STATUS_RESTATRT                       0x10
#define I2C_M_STATUS_TRANSMIT_ADR_ACK               0x18
#define I2C_M_STATUS_TRANSMIT_ADR_NACK              0x20
#define I2C_M_STATUS_TRANSMIT_DAT_ACK               0x28
#define I2C_M_STATUS_TRANSMIT_DAT_NCAK              0x30
#define I2C_M_STATUS_ARBITRATION_LOST               0x38
#define I2C_M_STATUS_RECEIVE_ADR_ACK                0x40
#define I2C_M_STATUS_RECEIVE_ADR_NACK               0x48
#define I2C_M_STATUS_RECEIVE_DAT_ACK                0x50
#define I2C_M_STATUS_RECEIVE_DAT_NACK               0x58
#define I2C_M_STATUS_BUS_ERROR                      0x00
#define I2C_M_STATUS_BUS_RELEASED                   0xF8
/**
  * @}
  */

/** @addtogroup I2C SLAVE Mode :Status Code
  * @{
  */
#define I2C_S_STAUTS_TRANSMIT_RESTATRT_OR_STOP      0xA0
#define I2C_S_STAUTS_TRANSMIT_ADR_ACK               0xA8
#define I2C_S_STAUTS_TRANSMIT_ARBITRATION_LOST      0xB0
#define I2C_S_STAUTS_TRANSMIT_DAT_ACK               0xB8
#define I2C_S_STAUTS_TRANSMIT_DAT_NACK              0xC0
#define I2C_S_STAUTS_TRANSMIT_LAST_DAT_ACK          0xC8
#define I2C_S_STAUTS_RECEIVE_ADR_ACK                0x60
#define I2C_S_STAUTS_RECEIVE_ARBIRATION_LOST        0x68
#define I2C_S_STAUTS_RECEIVE_DATA_ACK               0x80
#define I2C_S_STATUS_RECEIVE_DATA_NACK              0x88
#define I2C_S_STATUS_GC_ACK                         0x70
#define I2C_S_STATUS_GC_ARBIRATION_LOST             0x78
#define I2C_S_STATUS_GC_DATA_ACK                    0x90
#define I2C_S_STATUS_GC_DATA_NACK                   0x98
/**
  * @}
  */

HAL_StatusTypeDef i2c_pan2025_init(I2C_HandleTypeDef *hi2c);
HAL_StatusTypeDef i2c_pan2025_Mem_Write(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress , uint8_t *pData, uint16_t Size, uint32_t Timeout);
HAL_StatusTypeDef i2c_pan2025_Mem_Read(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size, uint32_t Timeout);
void HAL_I2C_IRQHandler(I2C_HandleTypeDef *hi2c);
HAL_StatusTypeDef i2c_pan2025_Mem_Write_IT(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size);
HAL_StatusTypeDef i2c_pan2025_Mem_Read_IT(I2C_HandleTypeDef *hi2c, uint16_t DevAddress, uint16_t MemAddress, uint8_t *pData, uint16_t Size);

#ifdef __cplusplus
}
#endif

#endif //_DRIVER_IIC_H_
