/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file rf.h
 * @brief PAN2025 series RF driver header file.
 *
 * @history - V0.01, 2019-03-25, yangyaming, first implementation.
*******************************************************************************/
#ifndef __RF_H
#define __RF_H

#ifdef __cplusplus
extern "C"{
#endif //__cplusplus

#include "pan2025xx.h"
 
#define     CE_HIGH                  RF->RF_CE = 1;
#define     CE_LOW                   RF->RF_CE = 0;

//RF Command
#define		R_RX_PAYLOAD		 (0x61)      //Read RX Payload
#define		W_TX_PAYLOAD		 (0xA0)      //Write TX Payload
#define		W_ACK_PAYLOAD		 (0xA8)      //Data with ACK
#define		W_TX_PAYLOAD_NOACK   (0xB0)      //TX Payload no ACK Request

#define		FLUSH_TX			 (0xE1)      //Flush RX FIFO
#define		FLUSH_RX			 (0xE2)      //Flush TX FIFO
#define		REUSE_TX_PL			 (0xE3)      //Reuse TX Payload
#define		ACTIVATE			 (0x73)      //ACTIVATE
#define		DEACTIVATE			 (0x8c)      //DEACTIVATE
#define		RST_HOLD		     (0x5a)      //
#define		RST_RELEASE	         (0xa5)      //
#define		NOP     	         (0xFF)      //

//interrupt status bit
#define     IRQ_RX_DR            (0x40)   // RX 接收完成标志位
#define     IRQ_TX_DS            (0x20)   // TX 发射完成标志位
#define     IRQ_RX_TX		     (0x60)   // TX/RX 完成标志位
#define     IRQ_MAX_RT           (0x10)   // 最大重传标志位
#define     IRQ_TX_FULL          (0x01)   // TX FIFO full 标志位
#define     IRQ_ALL_FLAGS        (0x70)

#define     DR_1M            (0x00)				//1Mbps
#define     DR_2M            (0x40)				//2Mbps
#define     DR_250K          (0xC0)             //250Kbps

#define     DSSS_EN          (0x01)
#define     DSSS_DSB         (0x00)

#define     RF_DR_1M_DSSS_EN           (0x0)
#define     RF_DR_1M_DSSS_DSB          (0x1)
#define     RF_DR_2M                   (0x2)
#define     RF_DR_250K                 (0x3)

/* RF power parameter*/
#define RF_POWER_DBM12	{0x00,0x80,0x70,0xfc,0x18}
#define RF_POWER_DBM10  {0x00,0x80,0x40,0xfc,0x18}
#define RF_POWER_DBM8	{0x00,0x80,0x30,0xf8,0x18}
#define RF_POWER_DBM6	{0x00,0x00,0x70,0xfc,0x18}
#define RF_POWER_DBM5	{0x00,0x00,0x60,0xfc,0x18}
#define RF_POWER_DBM4	{0x00,0x00,0x50,0xfc,0x18}
#define RF_POWER_DBM3	{0x00,0x00,0x30,0xfc,0x18}
#define RF_POWER_DBM2	{0x00,0x00,0x20,0xfc,0x18}
#define RF_POWER_DBM1	{0x00,0x00,0x20,0x5c,0x18}
#define RF_POWER_DBM0	{0x00,0x00,0x20,0x9c,0x00}
#define RF_POWER_DBM_1	{0x00,0x00,0x10,0xfc,0x18}
#define RF_POWER_DBM_2	{0x00,0x00,0x10,0x5c,0x18}
#define RF_POWER_DBM_6	{0x00,0x00,0x00,0x58,0x18}
#define RF_POWER_DBM_11	{0x00,0x00,0x00,0x10,0x18}

#define _RF_POWER_(dBm)	RF_POWER_##dBm

#define     TRANS_NORMAL_MODE           (1)
#define     TRANS_ENHANCED_MODE         (2)

#define ADDR_WITDTH_3_BYTE		1
#define ADDR_WITDTH_4_BYTE		2
#define ADDR_WITDTH_5_BYTE		3

#define DYN_MSK														1
#define NOACK_MSK													2
#define ACK_WITH_PAYLOAD_MSK										4
//#define ENHANCED_MODE1               (0)											//Disable DYN/NOACK/ACK_WITH_PAYLOAD
//#define ENHANCED_MODE2               (ACK_WITH_PAYLOAD)				//Enable 	ACK_WITH_PAYLOAD
//#define ENHANCED_MODE3               (NOACK)									//Enable	NOACK
//#define ENHANCED_MODE4               (DYN)										//Enable	DYN
//#define ENHANCED_MODE5               (DYN|ACK_WITH_PAYLOAD)		//Enable	DYN/ACK_WITH_PAYLOAD
//#define ENHANCED_MODE6               (DYN|NOACK)							//Enable	DYN/NOACK

#if defined(USER_DYN)
	#define DYN_FLAG									DYN_MSK
#else
	#define DYN_FLAG									0
#endif

#if defined(USER_NOACK)
	#define NOACK_FLAG								NOACK_MSK
#else
	#define	NOACK_FLAG								0
#endif

#if defined(USER_ACK_WITH_PAYLOAD)
	#define ACK_WITH_PAYLOAD_FLAG			ACK_WITH_PAYLOAD_MSK
#else
	#define ACK_WITH_PAYLOAD_FLAG			0
#endif

/* User configuration parameters */
#define ADDR_WITDTH				    		ADDR_WITDTH_5_BYTE
#define RF_CHANNEL                  (78) //2403
#define TX_PAYLOAD_LEN              (64)
#define RX_PAYLOAD_LEN              (64)
#define RF_DR                       (RF_DR_1M_DSSS_DSB) 
#define TX_ADDR_DEF								{0xcc,0xcc,0xcc,0xcc,0xcc}
#define RF_POWER									_RF_POWER_(DBM5)
#if defined(USER_ENHANCED)
	#define TRANSMIT_TYPE              (TRANS_ENHANCED_MODE)
#else
	#define TRANSMIT_TYPE              (TRANS_NORMAL_MODE)
#endif
#define FEATURE_MODE 				    (DYN_FLAG|NOACK_FLAG|ACK_WITH_PAYLOAD_FLAG)

void RF_Init(void);
void RF_ClockInit(void);
void RF_TxMode(void);
void RF_RxMode(void);

uint8_t RF_GetStatus(void);
void RF_ClearStatus(void);
void RF_ClearFIFO(void);
void RF_VccodeRead(void);
void RF_SetChannel(uint8_t chn);

void RF_SetRxLength(uint8_t length);
uint32_t RF_GetRxLength(void);
void RF_SetDataRate(uint8_t rate,uint8_t dsss);
void RF_WhiteningConfig(FunctionalState NewState);
void RF_ReadBuf(uint8_t cmd, uint8_t *pBuf,  uint8_t length);
void RF_WriteBuf(uint8_t cmd,uint8_t *pBuf, uint8_t length);
void RF_SetAddr(volatile uint32_t* reg, uint8_t* rf_addr_array, uint8_t addr_len);
void RF_ReadAddr(volatile uint32_t* reg, uint8_t* rf_addr_array, uint8_t addr_len);
void RF_Carrier( uint8_t channel);
void PowerSet(void);
#ifdef __cplusplus
}
#endif //__cplusplus

#endif //__RF_H
