/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file uart.h
 * @brief 
 *
 * @history - V0.01, 2019-03-20, yangyaming, first implementation.
*******************************************************************************/

#ifndef __UART_H
#define __UART_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "pan2025xx.h"

/*---------------------------------------------------------------------------------------------------------*/
/* UART_FIFO constants definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/

#define UART_FIFO_RFITL_1BYTE        (0x0 << UART_FIFO_RFITL_Pos)   /*!< UART_FIFO setting to set RX FIFO Trigger Level to 1 bit */
#define UART_FIFO_RFITL_2BYTES       (0x1 << UART_FIFO_RFITL_Pos)   /*!< UART_FIFO setting to set RX FIFO Trigger Level to 2 bits */
#define UART_FIFO_RFITL_4BYTES       (0x2 << UART_FIFO_RFITL_Pos)   /*!< UART_FIFO setting to set RX FIFO Trigger Level to 4 bits */
#define UART_FIFO_RFITL_6BYTES       (0x3 << UART_FIFO_RFITL_Pos)   /*!< UART_FIFO setting to set RX FIFO Trigger Level to 6 bits */

#define UART_FIFO_RTSTRGLV_1BYTE     (0x0 << UART_FIFO_RTSTRGLV_Pos)  /*!< UART_FIFO setting to set RTS Trigger Level to 1 bit */
#define UART_FIFO_RTSTRGLV_2BYTES    (0x1 << UART_FIFO_RTSTRGLV_Pos)  /*!< UART_FIFO setting to set RTS Trigger Level to 2 bits */
#define UART_FIFO_RTSTRGLV_4BYTES    (0x2 << UART_FIFO_RTSTRGLV_Pos)  /*!< UART_FIFO setting to set RTS Trigger Level to 4 bits */
#define UART_FIFO_RTSTRGLV_6BYTES    (0x3 << UART_FIFO_RTSTRGLV_Pos)  /*!< UART_FIFO setting to set RTS Trigger Level  to 6 bits */

/*---------------------------------------------------------------------------------------------------------*/
/* UART_LINE constants definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define UART_WORD_LEN_5        (0) /*!< UART_LINE setting to set UART word length to 5 bits */
#define UART_WORD_LEN_6        (1) /*!< UART_LINE setting to set UART word length to 6 bits */
#define UART_WORD_LEN_7        (2) /*!< UART_LINE setting to set UART word length to 7 bits */
#define UART_WORD_LEN_8        (3) /*!< UART_LINE setting to set UART word length to 8 bits */

#define UART_PARITY_NONE    (0x0 << UART_LINE_PBE_Pos) /*!< UART_LINE setting to set UART as no parity   */
#define UART_PARITY_ODD     (0x1 << UART_LINE_PBE_Pos) /*!< UART_LINE setting to set UART as odd parity  */
#define UART_PARITY_EVEN    (0x3 << UART_LINE_PBE_Pos) /*!< UART_LINE setting to set UART as even parity */
#define UART_PARITY_MARK    (0x5 << UART_LINE_PBE_Pos) /*!< UART_LINE setting to keep parity bit as '1'  */
#define UART_PARITY_SPACE   (0x7 << UART_LINE_PBE_Pos) /*!< UART_LINE setting to keep parity bit as '0'  */

#define UART_STOP_BIT_1     (0x0 << UART_LINE_NSB_Pos) /*!< UART_LINE setting for one stop bit  */
#define UART_STOP_BIT_1_5   (0x1 << UART_LINE_NSB_Pos) /*!< UART_LINE setting for 1.5 stop bit when 5-bit word length  */
#define UART_STOP_BIT_2     (0x1 << UART_LINE_NSB_Pos) /*!< UART_LINE setting for two stop bit when 6, 7, 8-bit word length */



/*@}*/ /* end of group PAN2025_UART_EXPORTED_CONSTANTS */


/** @addtogroup PAN2025_UART_EXPORTED_FUNCTIONS UART Exported Functions
  @{
*/



/**
 *    @brief    Write Data to Tx data register
 *
 *    @param[in]    uart    The base address of UART module.
 *    @param[in]    u8Data  Data byte to transmit 
 *                          
 *    @return    None
 */
__STATIC_INLINE void UART_Write(UART_T* uart, uint8_t u8Data)
{
    uart->DAT = u8Data;
}

/**
 *    @brief    Read Rx data register
 *
 *    @param[in]    uart        The base address of UART module. 
 *
 *    @return    The oldest data byte in RX FIFO 
 */
//#define UART_READ(uart)    (uart->DAT)
__STATIC_INLINE uint32_t UART_Read(UART_T* uart)
{
    return uart->DAT;
}

/**
 *    @brief    Get Tx empty register value. 
 *
 *    @param[in]   uart        The base address of UART module  
 *
 *    @return    Tx empty register value. 
 */
__STATIC_INLINE uint32_t UART_GetTxEmpty(UART_T* uart)
{
    return uart->FIFOSTS & UART_FIFOSTS_TXEMPTYF_Msk;
}

/**
 *    @brief    Get Rx empty register value. 
 *
 *    @param[in]    uart        The base address of UART module 
 * 
 *    @return    Rx empty register value.
 */
__STATIC_INLINE uint32_t UART_GetRxEmpty(UART_T* uart)
{
    return uart->FIFOSTS & UART_FIFOSTS_RXEMPTY_Msk;
}

/**
 *    @brief    Check specified uart port transmission is over. 
 *
 *    @param[in]    uart        The base address of UART module 
 * 
 *    @return    TE_Flag.
 */
__STATIC_INLINE uint32_t UART_IsTxEmpty(UART_T* uart)
{
    return (uart->FIFOSTS & UART_FIFOSTS_TXEMPTYF_Msk) >> UART_FIFOSTS_TXEMPTYF_Pos;
}

/**
 *    @brief    Wait specified uart port transmission is over 
 *
 *    @param[in]    uart        The base address of UART module 
 *
 *    @return    None 
 */
__STATIC_INLINE void UART_WaitTxEmpty(UART_T* uart)
{
    while(!(((uart->FIFOSTS) & UART_FIFOSTS_TXEMPTYF_Msk) >> UART_FIFOSTS_TXEMPTYF_Pos));
}

/**
 *    @brief    Check RDA_IF is set or not
 *
 *    @param[in]    uart        The base address of UART module 
 *
 *    @return     
 *            0 : The number of bytes in the RX FIFO is less than the RFITL  
 *            1 : The number of bytes in the RX FIFO equals or larger than RFITL
 */
__STATIC_INLINE uint32_t UART_IsRxReady(UART_T* uart)
{
    return (uart->INTSTS & UART_INTSTS_RDAIF_Msk)>>UART_INTSTS_RDAIF_Pos;
}

/**
 *    @brief    Check TX FIFO is full or not 
 *
 *    @param[in]    uart        The base address of UART module 
 *
 *    @return     
 *            1 = TX FIFO is full
 *            0 = TX FIFO is not full
 */
//#define UART_IS_TX_FULL(uart)    ((uart->FIFOSTS & UART_FIFOSTS_TXFULL_Msk)>>UART_FIFOSTS_TXFULL_Pos)
__STATIC_INLINE uint32_t UART_IsTxFull(UART_T* uart)
{
    return (uart->FIFOSTS & UART_FIFOSTS_TXFULL_Msk)>>UART_FIFOSTS_TXFULL_Pos;
}

/**
 *    @brief    Check RX FIFO is full or not 
 *
 *    @param[in]    uart        The base address of UART module  
 *
 *    @return
 *            1 = RX FIFO is full
 *            0 = RX FIFO is not full
 *
 */
__STATIC_INLINE uint32_t UART_IsRxFull(UART_T* uart)
{
    return (uart->FIFOSTS & UART_FIFOSTS_RXFULL_Msk)>>UART_FIFOSTS_RXFULL_Pos;
}

/**
 *    @brief    Get Tx full register value 
 *
 *    @param[in]    uart        The base address of UART module 
 *
 *    @return    Tx full register value 
 */
__STATIC_INLINE uint32_t UART_GetTxFull(UART_T* uart)
{
    return uart->FIFOSTS & UART_FIFOSTS_TXFULL_Msk;
}

/**
 *    @brief    Get Rx full register value 
 *
 *    @param[in]    uart        The base address of UART module 
 *
 *    @return    Rx full register value 
 */
__STATIC_INLINE uint32_t UART_GetRxFull(UART_T* uart)
{
    return uart->FIFOSTS & UART_FIFOSTS_RXFULL_Msk;
}

/**
 *    @brief    Enable specified interrupt
 *
 *    @param[in]    uart        The base address of UART module  
 *    @param[in]    u32eIntSel    Interrupt type select 
 *                        - \ref UART_INTEN_TOCNTEN_Msk        : Rx Time Out interrupt
 *                        - \ref UART_INTEN_BUFERRIEN_Msk      : Buffer Error interrupt   
 *                        - \ref UART_INTEN_RXTOIEN_Msk        : Rx time-out interrupt       
 *                        - \ref UART_INTEN_RLSIEN_Msk         : Rx Line status interrupt                              
 *                        - \ref UART_INTEN_THREIEN_Msk        : Tx empty interrupt          
 *                        - \ref UART_INTEN_RDAIEN_Msk         : Rx ready interrupt           
 *
 *    @return    None                                                    
 */
__STATIC_INLINE void UART_EnableInt(UART_T* uart, uint32_t u32eIntSel)
{
    uart->INTEN |= u32eIntSel;
}

/**
 *    @brief    Disable specified interrupt  
 *
 *    @param[in]    uart        The base address of UART module 
 *    @param[in]    u32eIntSel    Interrupt type select 
 *                        - \ref UART_INTEN_TOCNTEN_Msk        : Rx Time Out interrupt
 *                        - \ref UART_INTEN_BUFERRIEN_Msk      : Buffer Error interrupt   
 *                        - \ref UART_INTEN_RXTOIEN_Msk        : Rx time-out interrupt           
 *                        - \ref UART_INTEN_RLSIEN_Msk         : Rx Line status interrupt                              
 *                        - \ref UART_INTEN_THREIEN_Msk        : Tx empty interrupt          
 *                        - \ref UART_INTEN_RDAIEN_Msk         : Rx ready interrupt          
 *    @return    None                                                    
 */
__STATIC_INLINE void UART_DisableInt(UART_T* uart, uint32_t u32eIntSel)
{
    uart->INTEN &= ~u32eIntSel;
}



/**
 *    @brief    Set rx fifo interrupt trigger level
 *
 *    @param[in]    uart            The base address of UART module 
 *    @param[in]    level    Interrupt Type Flag,should be   
 *                            - \ref UART_FIFO_RFITL_1BYTE 
 *                            - \ref UART_FIFO_RFITL_2BYTES
 *                            - \ref UART_FIFO_RFITL_4BYTES
 *                            - \ref UART_FIFO_RFITL_6BYTES
 *
 *    @return       none                                                     
 */                                                                                      
__STATIC_INLINE void UART_SetRxFifoIntTriggerLevel(UART_T* uart, uint32_t level)
{
    uart->FIFO = (uart->FIFO & ~UART_FIFO_RFITL_Msk) | level;
}

/**
 *    @brief    judge interrupt occured or not
 *
 *    @param[in]    uart            The base address of UART module 
 *    @param[in]    intFlag         interrupt occured flag 
 *                                   UART_INTSTS_RDAIF_Msk     
 *                                   UART_INTSTS_THREIF_Msk      
 *                                   UART_INTSTS_RLSIF_Msk     
 *                                   UART_INTSTS_RXTOIF_Msk    
 *                                   UART_INTSTS_BUFERRIF_Msk  
 *                                   UART_INTSTS_RDAINT_Msk    
 *                                   UART_INTSTS_THREINT_Msk  
 *                                   UART_INTSTS_RLSINT_Msk  
 *                                   UART_INTSTS_RXTOINT_Msk 
 *                                   UART_INTSTS_BUFERRINT_Msk 
 *    @return       none                                                     
 */                                                                                      
__STATIC_INLINE uint32_t UART_IsIntOccured(UART_T* uart,uint32_t intFlag)
{
    return (uart->INTSTS & intFlag)?(1):(0);
}


/**
 *    @brief    judge fifo status matched or not
 *
 *    @param[in]    uart            The base address of UART module
 *    @param[in]    statusFlag      fifo status occured flag
 *                                   UART_FIFOSTS_RXOVIF_Msk
 *                                   UART_FIFOSTS_PEF_Msk
 *                                   UART_FIFOSTS_FEF_Msk
 *                                   UART_FIFOSTS_BIF_Msk
 *                                   UART_FIFOSTS_RXEMPTY_Msk
 *                                   UART_FIFOSTS_RXFULL_Msk
 *                                   UART_FIFOSTS_TXEMPTY_Msk
 *                                   UART_FIFOSTS_TXFULL_Msk
 *                                   UART_FIFOSTS_TXOVIF_Msk
 *                                   UART_FIFOSTS_TXEMPTYF_Msk
 *    @return       none
 */
__STATIC_INLINE uint32_t UART_IsFifoStatusMatched(UART_T* uart,uint32_t statusFlag)
{
    return (uart->FIFOSTS & statusFlag)?(1):(0);
}

/**
 *    @brief    clear fifo status flag
 *
 *    @param[in]    uart            The base address of UART module
 *    @param[in]    statusFlag      fifo status occured flag
 *                                   UART_FIFOSTS_RXOVIF_Msk
 *                                   UART_FIFOSTS_PEF_Msk
 *                                   UART_FIFOSTS_FEF_Msk
 *                                   UART_FIFOSTS_BIF_Msk
 *                                   UART_FIFOSTS_TXOVIF_Msk
 *    @return       none
 */
__STATIC_INLINE void UART_ClearFifoStatus(UART_T* uart,uint32_t statusFlag)
{
    uart->FIFOSTS  |= statusFlag;
}



void UART_Open(UART_T* uart, uint32_t u32baudrate);
void UART_Close(UART_T* uart );
void UART_ClearIntFlag(UART_T* uart , uint32_t u32InterruptFlag);
uint32_t UART_ReadMultipleData(UART_T* uart, uint8_t *pu8RxBuf, uint32_t u32ReadBytes);
uint32_t UART_WriteMultipleData(UART_T* uart,uint8_t *pu8TxBuf, uint32_t u32WriteBytes);
void UART_SetLineConfig(UART_T* uart, uint32_t u32baudrate, uint32_t u32data_width, uint32_t u32parity, uint32_t  u32stop_bits);
void UART_SetTimeoutCnt(UART_T* uart, uint32_t u32TOC);
void UART_ClearRxFifo(UART_T* uart);



#ifdef __cplusplus
}
#endif

#endif //__UART_H

