/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file wwdt.h
 * @brief 
 *
 * @history - V0.01, 2019-03-20, yangyaming, first implementation.
*******************************************************************************/

#ifndef __WWDT_H
#define __WWDT_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "pan2025xx.h" 

typedef enum _WWDT_Prescale
{
    WWDT_PRESCALER_1        = (0UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 1     \hideinitializer */
    WWDT_PRESCALER_2        = (1UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 2     \hideinitializer */
    WWDT_PRESCALER_4        = (2UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 4     \hideinitializer */
    WWDT_PRESCALER_8        = (3UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 8     \hideinitializer */
    WWDT_PRESCALER_16       = (4UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 16    \hideinitializer */
    WWDT_PRESCALER_32       = (5UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 32    \hideinitializer */
    WWDT_PRESCALER_64       = (6UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 64    \hideinitializer */
    WWDT_PRESCALER_128      = (7UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 128   \hideinitializer */
    WWDT_PRESCALER_192      = (8UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 192   \hideinitializer */
    WWDT_PRESCALER_256      = (9UL << WWDT_CTL_PSCSEL_Pos),   /*!< WWDT setting prescaler to 256   \hideinitializer */
    WWDT_PRESCALER_384      = (0xAUL << WWDT_CTL_PSCSEL_Pos), /*!< WWDT setting prescaler to 384   \hideinitializer */
    WWDT_PRESCALER_512      = (0xBUL << WWDT_CTL_PSCSEL_Pos), /*!< WWDT setting prescaler to 512   \hideinitializer */
    WWDT_PRESCALER_768      = (0xCUL << WWDT_CTL_PSCSEL_Pos), /*!< WWDT setting prescaler to 768   \hideinitializer */
    WWDT_PRESCALER_1024     = (0xDUL << WWDT_CTL_PSCSEL_Pos), /*!< WWDT setting prescaler to 1024  \hideinitializer */
    WWDT_PRESCALER_1536     = (0xEUL << WWDT_CTL_PSCSEL_Pos), /*!< WWDT setting prescaler to 1536  \hideinitializer */
    WWDT_PRESCALER_2048     = (0xFUL << WWDT_CTL_PSCSEL_Pos)  /*!< WWDT setting prescaler to 2048  \hideinitializer */
} WWDT_PrescaleDef;

#define WWDT_RELOAD_WORD          (0x00005AA5)                     ///< Fill this value to RLD register to reload WWDT counter  \hideinitializer 


/**
  * @brief This function clear WWDT time-out reset system flag.
  * @param None
  * @return None
  * \hideinitializer
  */
__STATIC_INLINE void WWDT_ClearResetFlag(void)
{
    WWDT->STATUS = WWDT_STATUS_WWDTRF_Msk;
}

/**
  * @brief This function clear WWDT compare match interrupt flag.
  * @param None
  * @return None
  * \hideinitializer
  */
__STATIC_INLINE void WWDT_ClearIntFlag(void)
{
    WWDT->STATUS = WWDT_STATUS_WWDTIF_Msk;
}

/**
  * @brief This function clear WWDT compare match event flag.
  * @param None
  * @return None
  * \hideinitializer
  */
__STATIC_INLINE void WWDT_ClearWWDTFFlag(void)
{
    WWDT->STATUS = WWDT_STATUS_WWDTF_Msk;
}

/**
  * @brief This function is use to get WWDT time-out reset system flag.
  * @return WWDT reset system or not
  * @retval false WWDT did not cause system reset
  * @retval true  WWDT caused system reset
  * \hideinitializer
  */
__STATIC_INLINE bool WWDT_GetResetFlag(void)
{
    return (bool)(WWDT->STATUS & WWDT_STATUS_WWDTRF_Msk);
}

/**
  * @brief This function is used to indicate WWDT compare match interrupt flag.
  * @return WWDT compare match interrupt occurred or not
  * @retval false WWDT compare match interrupt did not occur
  * @retval true  WWDT compare match interrupt occurred
  * \hideinitializer
  */
__STATIC_INLINE bool WWDT_GetIntFlag(void)
{
    return (bool)(WWDT->STATUS & WWDT_STATUS_WWDTIF_Msk);
}

/**
  * @brief This function is used to indicate WWDT compare match event flag.
  * @return WWDT compare match interrupt occurred or not
  * @retval false WWDT compare match interrupt did not occur
  * @retval true  WWDT compare match interrupt occurred
  * \hideinitializer
  */
__STATIC_INLINE bool WWDT_GetWWDTFFlag(void)
{
    return (bool)(WWDT->STATUS & WWDT_STATUS_WWDTF_Msk);
}

/**
  * @brief This function to reflects current WWDT counter value
  * @param None
  * @return Return current WWDT counter value
  * \hideinitializer
  */
__STATIC_INLINE uint32_t WWDT_GetCounter(void)
{
    return WWDT->CNT;
}

/**
  * @brief This function reflects current WWDT compare value (CMPDAT)
  * @param None
  * @return Return current WWDT compare value
  * \hideinitializer
  */
__STATIC_INLINE uint32_t WWDT_GetCompareValue(void)
{
    return (WWDT->CTL & WWDT_CTL_CMPDAT_Msk) >> WWDT_CTL_CMPDAT_Pos;
}

/**
  * @brief This function is used to reload the WWDT counter value to 0x3F.
  * @param None
  * @return None
  * @details After WWDT enabled, application must reload WWDT counter while
  *          current counter is less than compare value and larger than 0,
  *          otherwise WWDT will cause system reset.
  * \hideinitializer
  */
__STATIC_INLINE void WWDT_ReloadCounter(void)
{
    WWDT->RLDCNT  = WWDT_RELOAD_WORD;
}


/**
  * @brief This macro clear WWDT time-out reset system flag.
  * @param None
  * @return None
  * \hideinitializer
  */
#define WWDT_CLEAR_RESET_FLAG()  (WWDT->STATUS = WWDT_STATUS_WWDTRF_Msk)

/**
  * @brief This macro clear WWDT compare match interrupt flag.
  * @param None
  * @return None
  * \hideinitializer
  */
#define WWDT_CLEAR_INT_FLAG() (WWDT->STATUS = WWDT_STATUS_WWDTIF_Msk)

/**
  * @brief This macro clear WWDT compare match event flag.
  * @param None
  * @return None
  * \hideinitializer
  */
#define WWDT_CLEAR_WWDTF_FLAG() (WWDT->STATUS = WWDT_STATUS_WWDTF_Msk)

/**
  * @brief This macro is use to get WWDT time-out reset system flag.
  * @return WWDT reset system or not
  * @retval 0 WWDT did not cause system reset
  * @retval 1 WWDT caused system reset
  * \hideinitializer
  */
#define WWDT_GET_RESET_FLAG() (WWDT->STATUS & WWDT_STATUS_WWDTRF_Msk ? 1 : 0)

/**
  * @brief This macro is used to indicate WWDT compare match interrupt flag.
  * @return WWDT compare match interrupt occurred or not
  * @retval 0 WWDT compare match interrupt did not occur
  * @retval 1 WWDT compare match interrupt occurred
  * \hideinitializer
  */
#define WWDT_GET_INT_FLAG() (WWDT->STATUS & WWDT_STATUS_WWDTIF_Msk ? 1 : 0)

/**
  * @brief This macro is used to indicate WWDT compare match event flag.
  * @return WWDT compare match interrupt occurred or not
  * @retval 0 WWDT compare match interrupt did not occur
  * @retval 1 WWDT compare match interrupt occurred
  * \hideinitializer
  */
#define WWDT_GET_WWDTF_FLAG() (WWDT->STATUS & WWDT_STATUS_WWDTF_Msk ? 1 : 0)

/**
  * @brief This macro to reflects current WWDT counter value
  * @param None
  * @return Return current WWDT counter value
  * \hideinitializer
  */
#define WWDT_GET_COUNTER() (WWDT->CNT)

/**
  * @brief This macro is used to reload the WWDT counter value to 0x3F.
  * @param None
  * @return None
  * @details After WWDT enabled, application must reload WWDT counter while
  *          current counter is less than compare value and larger than 0,
  *          otherwise WWDT will cause system reset.
  * \hideinitializer
  */
#define WWDT_RELOAD_COUNTER() (WWDT->RLDCNT  = WWDT_RELOAD_WORD)

void WWDT_Open(WWDT_PrescaleDef preScale, uint32_t u32CmpValue, uint32_t u32EnableInt);
void WWDT_Close(void);
void WWDT_SelectClockSource(uint32_t u32clksel);

/*@}*/ /* end of group PN501_WWDT_EXPORTED_FUNCTIONS */



#ifdef __cplusplus
}
#endif

#endif //__WWDT_H

