/*******************************************************************************
 * @note Copyright (C) 2020 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file clk.c
 * @brief 
 *
 * @history - V0.01, 2020-03-19, yangyaming, first implementation.
                     2020-04-07  CLK_RefClkSrcConfig;
                                 ޸CLK_SYSCLKConfig                     
*******************************************************************************/
#include "clk.h"

void CLK_delay_10nop(uint32_t u32NopCnt)
{
	while(u32NopCnt--)
	{
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
	}
}
/**
  * @brief  This function clear the selected system reset source
  * @param[in]  u32Src is system reset source
  * @return None
  */
void CLK_ClearResetSrc(uint32_t u32Src)
{
    CLK->RSTSTS |= u32Src;
}

/**
  * @brief  This function get Brown-out detector output status
  * @return 0: System voltage is higher than BOD_VL setting or BOD_EN is 0.
  *         1: System voltage is lower than BOD_VL setting.
  *         Note : If the BOD_EN is 0, this function always return 0.
  */
uint32_t CLK_GetBODStatus(void)
{
    return (CLK->BODCTL & CLK_BODCTL_BODOUT_Msk)?1:0;
}

/**
  * @brief  This function get the system reset source register value
  * @return Reset source
  */
uint32_t CLK_GetResetSrc(void)
{
    return (CLK->RSTSTS);
}



/**
  * @brief  This function reset chip.
  * @return None
  */
void CLK_ResetChip(void)
{
    CLK->IPRST0 |= CLK_IPRST0_CHIPRST_Msk;
}

/**
  * @brief  This function reset CPU.
  * @return None
  */
void CLK_ResetCPU(void)
{
    CLK->IPRST0 |= CLK_IPRST0_CPURST_Msk;
}

/**
  * @brief  This function reset selected modules.
  * @param[in]  u32ModuleIndex is module index. Including :
  *                  - \ref ADC_RST
  *                  - \ref ACMP_RST
  *                  - \ref PWM0_RST
  *                  - \ref WWDT_RST
  *                  - \ref WDT_RST
  *                  - \ref UART1_RST
  *                  - \ref UART0_RST
  *                  - \ref SPI1_RST
  *                  - \ref SPI0_RST
  *                  - \ref I2C0_RST
  *                  - \ref I2C1_RST
  *                  - \ref TMR2_RST
  *                  - \ref TMR1_RST
  *                  - \ref TMR0_RST
  *                  - \ref GPIO_RST
  *	@param[in]  u32HoldTime is Reset time hold time. It should be equals the clock of the module.
  * @return None
  */

void SYS_ResetModule(uint32_t u32ModuleIndex, uint32_t u32HoldTime)
{
    *(volatile uint32_t *)((uint32_t)&(CLK->IPRST0) + (u32ModuleIndex>>24)) |= 1<<(u32ModuleIndex & 0x00ffffff);
	CLK_delay_10nop(u32HoldTime);
    *(volatile uint32_t *)((uint32_t)&(CLK->IPRST0) + (u32ModuleIndex>>24)) &= ~(1<<(u32ModuleIndex & 0x00ffffff));
	CLK_delay_10nop(u32HoldTime);
}




/**
  * @brief  This function configure BOD function.
  *         Configure BOD reset or interrupt mode and set Brown-out voltage level.
  *         Enable Brown-out function
  * @param[in]  i32Mode is reset or interrupt mode. Including :
  *                  - \ref CLK_BODCTL_BOD_RST_EN
  *                  - \ref CLK_BODCTL_BOD_INTERRUPT_EN
  * @param[in]  u32BODLevel is Brown-out voltage level. Including :
  *                  - \ref CLK_BODCTL_DISABLE
  *                  - \ref CLK_BODCTL_BODVL_2_67V
  *                  - \ref CLK_BODCTL_BODVL_2_4V
  *                  - \ref CLK_BODCTL_BODVL_2_15V
  * @return None
  */
void CLK_EnableBOD(int32_t i32Mode, uint32_t u32BODLevel)
{

    CLK->BODCTL = (CLK->BODCTL & ~CLK_BODCTL_BODRSTEN_Msk) | i32Mode;
}

/**
  * @brief  This function disable BOD function.
  * @return None
  */
void SYS_DisableBOD(void)
{
}

/**
  * @brief  This function get HCLK frequency. The frequency unit is Hz.
  * @return HCLK frequency
  */
uint32_t CLK_GetHCLKFreq(void)
{
    SystemCoreClockUpdate();
    return SystemCoreClock;
}


 /**
   * @brief  This function get CPU frequency. The frequency unit is Hz.
   * @return CPU frequency
   */
 uint32_t CLK_GetCPUFreq(void)
 {
     SystemCoreClockUpdate();
     return SystemCoreClock;
 }
 
 /**
   * @brief  This function get APB1 frequency. The frequency unit is Hz.
   * @return CPU frequency
   */ 
uint32_t CLK_GetPCLK1Freq(void)
{
    uint32_t apb1_div = 0, apb1_freq;
    uint32_t tmpreg = 0;

    tmpreg = CLK->CLK_DIV_1;
    tmpreg &= CLK_CLK_DIV1_APB1_DIV_Msk;
    apb1_div = (tmpreg >> CLK_CLK_DIV1_APB1_DIV_Pos);
    
    SystemCoreClockUpdate();
    
    //2N divider
    if(apb1_div)
        apb1_freq = SystemCoreClock / (apb1_div * 2);
    else
        apb1_freq = SystemCoreClock;
    
    return apb1_freq;
}

 /**
   * @brief  This function get APB2 frequency. The frequency unit is Hz.
   * @return CPU frequency
   */ 
uint32_t CLK_GetPCLK2Freq(void)
{
    uint32_t apb2_div = 0, apb2_freq;
    uint32_t tmpreg = 0;

    tmpreg = CLK->CLK_DIV_1;
    tmpreg &= CLK_CLK_DIV1_APB2_DIV_Msk;
    apb2_div = (tmpreg >> CLK_CLK_DIV1_APB2_DIV_Pos);
    
    SystemCoreClockUpdate();
    
    //2N divider
    if(apb2_div)
        apb2_freq = SystemCoreClock / (apb2_div * 2);
    else
        apb2_freq = SystemCoreClock;
    
    return apb2_freq;
}

/**
  * @brief  This function get MCU DPLL Freq.
  * @param None
  * @return None
  */
uint32_t CLK_GetPLLClockFreq(void)
{
    uint32_t dpll_freq = FREQ_48MHZ;
    uint32_t u32PllReg;
    
    u32PllReg = CLK->CLK_AHB_SEL;
    switch((u32PllReg & CLK_DIGITAL_CLK_SEL_Msk) >> CLK_DIGITAL_CLK_SEL_Pos)
    {
       case CLK_DPLL_CLKSEL_48M:
           dpll_freq = FREQ_48MHZ;
           break;
       case CLK_DPLL_CLKSEL_57M_6:
           dpll_freq = FREQ_57M6HZ;
           break;
       case CLK_DPLL_CLKSEL_72M:
       case CLK_DPLL_CLKSEL_72M_CASE1:            
           dpll_freq = FREQ_72MHZ;
           break;
       default:
           break;
    }
   return dpll_freq;
}

/**
  * @brief  This function set MCU DPLL Freq.
  * @param dpll_sel select dpll output
  *        - \ref CLK_DPLL_CLKSEL_48M
  *        - \ref CLK_DPLL_CLKSEL_57M_6
  *        - \ref CLK_DPLL_CLKSEL_72M
  * @return None
  */
void CLK_SetPLLClockFreq(uint32_t dpll_sel)
{
    uint32_t tmpreg;
    
    tmpreg = CLK->CLK_AHB_SEL;
    
    tmpreg &= ~(CLK_DIGITAL_CLK_SEL_Msk);
    tmpreg |= (dpll_sel << CLK_DIGITAL_CLK_SEL_Pos) & CLK_DIGITAL_CLK_SEL_Msk;
    
    SYS_UnlockReg();
    CLK->CLK_AHB_SEL = tmpreg;
    SYS_LockReg();
}

/**
    * @brief  This function set 16M ref clock source
    * @param[in]  u32ClkSrc is HCLK clock source. Including :
    *                  - \ref CLK_AHB_CLKSEL_16M_RC
    *                  - \ref CLK_AHB_CLKSEL_16M_HXT
    * @return None
    */
 void CLK_RefClkSrcConfig(uint32_t u32ClkSrc)
 {
    uint32_t tmpreg;
    
    switch(u32ClkSrc){
        case CLK_AHB_CLKSEL_16M_RC:
            break;
        case CLK_AHB_CLKSEL_16M_HXT:
            // enable XTAL
            ANAC->LP_FL_CTRL1|= (PWR_UP_Msk | CE_INT_Msk);
            CLK_Wait3vSyncReady();
            CLK_WaitClockReady(CLK_CLK_STATUS_16M_XTL_STB_Msk);
            break;
        default:
            break;
    }
    tmpreg = CLK->CLK_AHB_SEL;
    tmpreg &= ~CLK_CLK_AHB_SEL_Msk;
    tmpreg |= (u32ClkSrc << CLK_CLK_AHB_SEL_Pos) & CLK_CLK_AHB_SEL_Msk;
    
    SYS_UnlockReg();
    /* Switch HCLK to new HCLK source */
    CLK->CLK_AHB_SEL = tmpreg;
    SYS_LockReg();

    CLK_Wait3vSyncReady();
    if(CLK_AHB_16M_SRC_CLKSEL_HXT == u32ClkSrc){
        CLK_WaitClockReady(CLK_CLK_STATUS_16M_XTL_STB_Msk);
    }        
 }
 
 /**
    * @brief  This function set HCLK clock source and HCLK clock divider
    * @param[in]  u32ClkSrc is HCLK clock source. Including :
    *                  - \ref CLK_AHB_CLKSEL_16M_RC
    *                  - \ref CLK_AHB_CLKSEL_16M_HXT
    *                  - \ref CLK_AHB_CLKSEL_MCU_DPLL
    * @return None
    */
void CLK_SYSCLKConfig(uint32_t u32ClkSrc)
{
    uint32_t clkDpllSelect;
    if(CLK_AHB_CLKSEL_MCU_DPLL == u32ClkSrc){
        
        ANAC->LP_FL_CTRL1 |= EN_LDO_DPLL_Msk;   //EN_LDO_DPLL
        CLK_Wait3vSyncReady();
        //DPLL reference clock selection
        ANAC->LP_MISC |= DPLL_CLK_SEL_Msk;
        CLK_Wait3vSyncReady();
        ANAC->LP_FL_CTRL1 |= EN_DPLL_Msk;
        CLK_Wait3vSyncReady();
        CLK_WaitClockReady(CLK_CLK_STATUS_MCU_PLL_STB_Msk);
        clkDpllSelect = (CLK->CLK_AHB_SEL & CLK_DIGITAL_CLK_SEL_Msk) >> CLK_DIGITAL_CLK_SEL_Pos;
        if((CLK_DPLL_CLKSEL_72M == clkDpllSelect) 
            || (CLK_DPLL_CLKSEL_72M_CASE1 == clkDpllSelect)
            || (CLK_DPLL_CLKSEL_57M_6 == clkDpllSelect)
        ){
            SYS_UnlockReg();
            FMC->FATCTL = 0x2;
            SYS_LockReg();
        }
    }
        
    SYS_UnlockReg();
    /* Switch HCLK to new HCLK source */
    CLK->CLK_AHB_SEL = (CLK->CLK_AHB_SEL & ~CLK_CLK_AHB_SEL_Msk) | u32ClkSrc;
    //SYS_LockReg();

    /* Update System Core Clock */
    SystemCoreClockUpdate();
}

/**
  * @brief  Configures the AHB clock (HCLK).
  * @note   Depending on the device voltage range, the software has to set correctly
  *         these bits to ensure that HCLK not exceed the maximum allowed frequency
  *         (for more details refer to section above
  *           "CPU, AHB and APB busses clocks configuration functions")
  * @param  CLK_SYSCLK: defines the AHB clock divider. This clock is derived from 
  *         the system clock (SYSCLK).
  *          This parameter can be one of the following values:
  *            @arg CLK_SYSCLK_Div1: AHB clock = SYSCLK
  *            @arg CLK_SYSCLK_Div2: AHB clock = SYSCLK/2
  *            @arg CLK_SYSCLK_Div4: AHB clock = SYSCLK/4
  *            @arg CLK_SYSCLK_Div8: AHB clock = SYSCLK/8
  *            @arg CLK_SYSCLK_Div16: AHB clock = SYSCLK/16
  * @retval None
  */
void CLK_HCLKConfig(uint32_t CLK_SYSCLK)
{
    uint32_t tmpreg = 0;

    /* Check the parameters */
    assert_param(IS_CLK_HCLK(CLK_SYSCLK));

    tmpreg = CLK->CLK_DIV_0;

    /* Clear HPRE[3:0] bits */
    tmpreg &= ~CLK_CLK_DIV0_AHB_DIV_Msk;

    /* Set divier[3:0] bits according to CLK_SYSCLK value */
    tmpreg |= CLK_SYSCLK;

    /* Store the new value */
    CLK->CLK_DIV_0 = tmpreg;
}
/**
  * @brief  Configures the Low Speed APB clock (PCLK1).
  * @param  CLK_HCLK: defines the APB1 clock divider. This clock is derived from 
  *         the AHB clock (HCLK).
  *          This parameter can be one of the following values:
  *            @arg CLK_HCLK_APB1_Div1:  APB1 clock = HCLK
  *            @arg CLK_HCLK_APB1_Div2:  APB1 clock = HCLK/2
  *            @arg CLK_HCLK_APB1_Div4:  APB1 clock = HCLK/4
  *            @arg CLK_HCLK_APB1_Div8:  APB1 clock = HCLK/8
  *            @arg CLK_HCLK_APB1_Div16: APB1 clock = HCLK/16
  * @retval None
  */
void CLK_PCLK1Config(uint32_t u32ClkDiv )
{
    uint32_t tmpreg = 0;

    tmpreg = CLK->CLK_DIV_1;

    /* Clear PPRE1[2:0] bits */
    tmpreg &= ~CLK_CLK_DIV1_APB1_DIV_Msk;

    /* Set CLK_DIV_2[23:16] bits according to CLK_HCLK value */
    tmpreg |= u32ClkDiv;

    /* Store the new value */
    CLK->CLK_DIV_1 = tmpreg;
}

/**
  * @brief  Configures the High Speed APB clock (PCLK2).
  * @param  CLK_HCLK: defines the APB2 clock divider. This clock is derived from 
  *         the AHB clock (HCLK).
  *          This parameter can be one of the following values:
  *            @arg CLK_HCLK_APB2_Div1:  APB2 clock = HCLK
  *            @arg CLK_HCLK_APB2_Div2:  APB2 clock = HCLK/2
  *            @arg CLK_HCLK_APB2_Div4:  APB2 clock = HCLK/4
  *            @arg CLK_HCLK_APB2_Div8:  APB2 clock = HCLK/8
  *            @arg CLK_HCLK_APB2_Div16: APB2 clock = HCLK/16
  * @retval None
  */
void CLK_PCLK2Config(uint32_t u32ClkDiv)
{
    uint32_t tmpreg = 0;

    tmpreg = CLK->CLK_DIV_1;

    tmpreg &= ~CLK_CLK_DIV1_APB2_DIV_Msk;

    /* Set CLK_DIV_2[31:24] bits according to CLK_HCLK value */
    tmpreg |= u32ClkDiv;

    /* Store the new value */
    CLK->CLK_DIV_1 = tmpreg;

}


/**
  * @brief  Enables or disables the AHB2 peripheral clock.
  * @note   After reset, the peripheral clock (used for registers read/write access)
  *         is disabled and the application software has to enable this clock before 
  *         using it. 
  * @param  CLK_AHBPeriph: specifies the AHB2 peripheral to gates its clock.
  *          This parameter can be any combination of the following values:
  *            @arg CLK_AHBPeriph_RF:   RF clock
  *            @arg CLK_AHBPeriph_SysTick: SysTick clock
  * @param  NewState: new state of the specified peripheral clock.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CLK_AHBPeriphClockCmd(uint32_t CLK_AHBPeriph, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_CLK_AHB2_PERIPH(CLK_AHBPeriph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        CLK->CLK_AHB_EN |= CLK_AHBPeriph;
    }
    else
    {
        CLK->CLK_AHB_EN &= ~CLK_AHBPeriph;
    }
}

/**
  * @brief  Enables or disables the Low Speed APB (APB1) peripheral clock.
  * @note   After reset, the peripheral clock (used for registers read/write access)
  *         is disabled and the application software has to enable this clock before 
  *         using it. 
  * @param  CLK_APB1Periph: specifies the APB1 peripheral to gates its clock.
  *          This parameter can be any combination of the following values:
  *            @arg CLK_APB1Periph_I2C0     I2C0 Clock               
  *            @arg CLK_APB1Periph_SPI0    SPI0 Clock       
  *            @arg CLK_APB1Periph_UART0    UART0 Clock     
  *            @arg CLK_APB1Periph_PWM01    PWM01 Clock       
  *            @arg CLK_APB1Periph_PWM23    PWM23 Clock   
  *            @arg CLK_APB1Periph_PWM45    PWM45 Clock    
  *            @arg CLK_APB1Periph_PWM67    PWM67 Clock     
  *            @arg CLK_APB1Periph_ADC      ADC Clock     
  *            @arg CLK_APB1Periph_WDT      WDT Clock    
  *            @arg CLK_APB1Periph_WWDT     WWDT Clock    
  *            @arg CLK_APB1Periph_TMR0     TMR0 Clock     
  *            @arg CLK_APB1Periph_APB1     APB1 Clock     
  * @param  NewState: new state of the specified peripheral clock.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CLK_APB1PeriphClockCmd(uint32_t CLK_APB1Periph, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_CLK_APB1_PERIPH(CLK_APB1Periph));  
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    CLK->CLK_APB1_EN |= CLK_APB1Periph;
  }
  else
  {
    CLK->CLK_APB1_EN &= ~CLK_APB1Periph;
  }
}

/**
  * @brief  Enables or disables the High Speed APB (APB2) peripheral clock.
  * @note   After reset, the peripheral clock (used for registers read/write access)
  *         is disabled and the application software has to enable this clock before 
  *         using it.
  * @param  CLK_APB2Periph: specifies the APB2 peripheral to gates its clock.
  *          This parameter can be any combination of the following values:
  *            @arg CLK_APB2Periph_I2C1      I2C1 Clock
  *            @arg CLK_APB2Periph_SPI1     SPIM1 Clock
  *            @arg CLK_APB2Periph_UART1     UART1 Clock
  *            @arg CLK_APB2Periph_TMR1      TMR1 Clock
  *            @arg CLK_APB2Periph_TMR2      TMR2 Clock
  *            @arg CLK_APB2Periph_UART2     UART2 Clock
  *            @arg CLK_APB2Periph_UART3     UART3 Clock
  *            @arg CLK_APB2Periph_TMR3      TMR3 Clock
  *            @arg CLK_APB2Periph_TMR4      TMR4 Clock
  *            @arg CLK_APB2Periph_APB2      APB2 Clock
  * @param  NewState: new state of the specified peripheral clock.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CLK_APB2PeriphClockCmd(uint32_t CLK_APB2Periph, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_CLK_APB2_PERIPH(CLK_APB2Periph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState != DISABLE)
    {
        CLK->CLK_APB2_EN |= CLK_APB2Periph;
    }
    else
    {
        CLK->CLK_APB2_EN &= ~CLK_APB2Periph;
    }
}


 /**
   * @brief      Disable System Tick counter
   * @return     None
   * @details    This function disable System Tick counter.
   */
 void CLK_DisableSysTick(void)
 {
     SysTick->CTRL = 0;    /* Set System Tick counter disabled */
 }


 /**
   * @brief  This function execute delay function.
   * @param[in]  us  Delay time. The Max value is 2^24 / CPU Clock(MHz). Ex:
   *                             50MHz => 335544us, 48MHz => 349525us, 28MHz => 699050us ...
   * @return None
   * @details    Use the SysTick to generate the delay time and the UNIT is in us.
   *             The SysTick clock source is from HCLK, i.e the same as system core clock.
   */
 void CLK_SysTickDelay(uint32_t us)
 {
     SysTick->LOAD = us * CyclesPerUs;
     SysTick->VAL  =  (0x00);
     SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk;

     /* Waiting for down-count to zero */
     while((SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk) == 0);
     SysTick->CTRL = 0;
 }

 /**
   * @brief  This function check selected clock source status
   * @param[in]  u32ClkMask is selected clock source. Including
   *                  - \ref CLK_CLK_STATUS_32K_RC_STB_Msk
   *                  - \ref CLK_CLK_STATUS_16M_RC_STB_Msk
   *                  - \ref CLK_CLK_STATUS_16M_XTL_STB_Msk
   *                  - \ref CLK_CLK_STATUS_MCU_PLL_STB_Msk
   *                  - \ref CLK_CLK_STATUS_RF_PLL_STB_Msk
   *                  - \ref CLK_CLK_STATUS_32K_XTL_STB_Msk
   * @return   0  clock is not stable
   *           1  clock is stable
   *
   * @details  To wait for clock ready by specified CLKSTATUS bit or timeout (~5ms)
   */
 uint32_t CLK_WaitClockReady(uint32_t u32ClkMask)
 {
     int32_t i32TimeOutCnt = 2160000;

     while((CLK->CLK_STB_STATUS & u32ClkMask) != u32ClkMask) {
         if(i32TimeOutCnt-- <= 0)
             return 0;
     }
     return 1;
 }
 
 /**
   * @brief  This function wait sync 3v clock locale stable
   * @param[in]  none
   * @return   0  clock sync is not stable
   *           1  clock sync is stable
   *
   * @details  To wait for clock ready by specified CLKSTATUS bit or timeout (~5ms)
   */
 uint32_t CLK_Wait3vSyncReady(void)
 {
     int32_t i32TimeOutCnt = 2160000;

    ANAC->LP_REG_SYNC |= REG_SYNC_3V_Msk;
    while(ANAC->LP_REG_SYNC & (REG_SYNC_STS_Msk | REG_SYNC_3V_Msk))
    {
         if(i32TimeOutCnt-- <= 0)
             return 0;
    }

     return 1;
 }

/**
  * @brief This API is used to select wdt clock source
  * @param[in] u32clksel wdt clock selection. Could be 
  *             - \ref CLK_APB_WDTSEL_APB1_DIV2048, \ref CLK_APB_WDTSEL_LIRC
  * @return none
  * @note This API is only used to select wdt clock source
  */

void CLK_SetWdtClkSrc(uint32_t u32clksel)
{
    CLK->CLK_APB_SEL = (CLK->CLK_APB_SEL & ~CLK_APB_WDTSEL_Msk) | u32clksel;
    return;
}

/**
  * @brief This API is used to select wwdt clock source
  * @param[in] wwdt The base address of wwdt module
  * @param[in] u32clksel wwdt clock selection. Could be 
  *             - \ref CLK_APB_WWDTSEL_APB1_DIV2048, \ref CLK_APB_WWDTSEL_LIRC
  * @return none
  * @note This API is only used to select wwdt clock source
  */

void CLK_SetWwdtClkSrc(uint32_t u32clksel)
{
    CLK->CLK_APB_SEL = (CLK->CLK_APB_SEL & ~CLK_APB_WWDTSEL_Msk) | u32clksel;
    return;
}

/**
  * @brief This API is used to SELECT timer clock source
  * @param[in] timer The base address of Timer module
  * @param[in] u32clksel timer clock selection. Could be 
  *             - \ref CLK_APB_TMR0SEL_APB1CLK, \ref CLK_APB_TMR0SEL_LIRC,\ref CLK_APB_TMR0SEL_TM0
  *             - \ref CLK_APB_TMR1SEL_APB2CLK, \ref CLK_APB_TMR1SEL_LIRC,\ref CLK_APB_TMR0SEL_TM1
  *             - \ref CLK_APB_TMR2SEL_APB2CLK, \ref CLK_APB_TMR2SEL_LIRC,\ref CLK_APB_TMR0SEL_TM2
  * @return none
  * @note This API is only used to select timer clock source
  */

void CLK_SetTmrClkSrc(TIMER_T *timer, uint32_t u32clksel)
{
    if(timer == TIMER0) {        
        CLK->CLK_APB_SEL = (CLK->CLK_APB_SEL & ~CLK_APB_TMR0SEL_Msk) | u32clksel;
    }
    else if(timer == TIMER1){
        CLK->CLK_APB_SEL = (CLK->CLK_APB_SEL & ~CLK_APB_TMR1SEL_Msk) | u32clksel;
    }
    else if(timer == TIMER2){
        CLK->CLK_APB_SEL = (CLK->CLK_APB_SEL & ~CLK_APB_TMR2SEL_Msk) | u32clksel;
    }
}


void CLK_AdcReset(void)
{
    CLK->IPRST1 |= CLK_IPRST1_ADCRST_Msk;
    CLK->IPRST1 &= ~CLK_IPRST1_ADCRST_Msk;
}

uint32_t CLK_GetPeripheralFreq(void * Peripheral)
{
    uint8_t u8ApbDiv;
	uint32_t u32AhbClk,u32Pclk;

	/*get system clock frequency*/
    u32AhbClk = CLK_GetHCLKFreq(); 
    
    if((I2C0    == (I2C_T *)   Peripheral) 
     ||(UART0   == (UART_T *)  Peripheral)
     ||(SPI0    == (SPI_T *)   Peripheral)
     ||(TIMER0  == (TIMER_T *) Peripheral))
    {
        u8ApbDiv = (CLK->CLK_DIV_1 & CLK_CLK_DIV1_APB1_DIV_Msk) >> CLK_CLK_DIV1_APB1_DIV_Pos;
    }
    else if((UART1   == (UART_T *)  Peripheral) 
          ||(UART2   == (UART_T *)  Peripheral)
          ||(UART3   == (UART_T *)  Peripheral)
          ||(TIMER1  == (TIMER_T *) Peripheral)
          ||(TIMER2  == (TIMER_T *) Peripheral))
        u8ApbDiv = (CLK->CLK_DIV_1 & CLK_CLK_DIV1_APB2_DIV_Msk) >> CLK_CLK_DIV1_APB2_DIV_Pos;
    
    if(u8ApbDiv)
        u32Pclk = u32AhbClk / (2 * u8ApbDiv);
    else
        u32Pclk = u32AhbClk;
    return u32Pclk;
}

void CLK_EnableTempDetect(void)
{
    ANAC->ANA_1P5V_RSV |= EN_TEMP_Msk;
}

void CLK_DisableTempDetect(void)
{
    ANAC->ANA_1P5V_RSV &= ~EN_TEMP_Msk;
}

