/*******************************************************************************
 * @note Copyright (C) 2020 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file fmc.c
 * @brief PAN2025 series FMC driver source file.
 *
 * @history - V0.01, 2020-03-19, yangyaming, first implementation.
*******************************************************************************/
#include "fmc.h"

 /**
 * @brief      Enable flash ISP opertion
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_Open(void)
{
    FMC->ISPCTL |=  FMC_ISPCTL_ISPEN_Msk;
}


 /**
 * @brief      Disable flash ISP opertion
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_Close(void)
{
    FMC->ISPCTL &= ~FMC_ISPCTL_ISPEN_Msk;
}


/**
 * @brief      Read company ID
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_ReadCID(void)
{
    FMC->ISPCMD = FMC_ISPCMD_READ_CID;
    FMC->ISPADDR = 0x0;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

		return FMC->ISPDAT;

}

/**
 * @brief      Read product ID
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_ReadPID(void)
{
    FMC->ISPCMD = FMC_ISPCMD_READ_PID;
    FMC->ISPADDR = 0x04;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;
    return FMC->ISPDAT;
}


/**
 * @brief      Read unique ID
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_ReadUID(uint32_t index)
{
    FMC->ISPCMD = FMC_ISPCMD_READ_UID;
    FMC->ISPADDR = index * 4;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;

    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    return FMC->ISPDAT;
}


/**
 * @brief      Read unique company ID
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_ReadUCID(uint32_t index)
{
    FMC->ISPCMD = FMC_ISPCMD_READ_UID;
    FMC->ISPADDR = index * 4 + 0x10;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;

    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    return FMC->ISPDAT;
}


 /**
 * @brief      Flash page erase
 * @param[in]  u32PageAddr : page address for page to be erased, must be 512Byte alligen
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
int32_t FMC_Erase(uint32_t u32PageAddr)
{
    FMC->ISPCMD = FMC_ISPCMD_PAGE_ERASE;
    FMC->ISPADDR = u32PageAddr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;

    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    if(FMC->ISPCTL & FMC_ISPCTL_ISPFF_Msk){
        FMC->ISPCTL |= FMC_ISPCTL_ISPFF_Msk;
        return -1;
    }
    return 0;
}


 /**
 * @brief      write one word of data to assigned address in flash  
 * @param[in]  u32Addr: address in flash to write data
 * @return     pdata : buffer to reserve data to be written to flash
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_Write(uint32_t u32Addr, uint32_t *u32Data)
{
    FMC->ISPCMD = FMC_ISPCMD_PROGRAM;
    FMC->ISPADDR = u32Addr;
    FMC->ISPDAT = *u32Data;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;
}

 /**
 * @brief      Read one word  data from assigned address  
 * @param[in]  u32Addr: address in flash to read data
 * @return     pdata : buffer to reserve data from flash
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_Read(uint32_t u32Addr, uint32_t *pdata)
{

    FMC->ISPCMD = FMC_ISPCMD_READ;
    FMC->ISPADDR = u32Addr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;

	
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;
    
    *pdata = FMC->ISPDAT;

  	return 0;
}


 /**
 * @brief      set system memory vector address 
 * @param[in]  u32Addr: address will be mapped to system memory vector
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_SetVectorPageAddr(uint32_t u32PageAddr)
{
    FMC->ISPCMD = FMC_ISPCMD_VECMAP;
    FMC->ISPADDR = u32PageAddr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;
}


 /**
 * @brief      Flash page erase
 * @param[in]  u32PageAddr : page address for page to be erased, must be 512Byte alligen
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_GetVectorPageAddr(void)
{
    return (FMC->ISPSTS & 0x0FFFFF00ul);
}


 /**
 * @brief      Flash page erase
 * @param[in]  u32PageAddr : page address for page to be erased, must be 512Byte alligen
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_Page_Erase(uint32_t u32Addr)
{
  FMC->ISPCMD = FMC_ISPCMD_PAGE_ERASE;                         
  FMC->ISPADDR = u32Addr;           										
  FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;                             
  while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) {                    
  }
}



/**
 * @brief      Flash chip erase
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
void FMC_ChipErase(void)
{
  FMC->ISPCMD = FMC_ISPCMD_CHIP_ERASE;                       
  FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;                        
  while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) {               
  }
}


 /**
 * @brief      Get data flash base address 
 * @param[in]  none
 * @return     data flash base address
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t FMC_ReadDataFlashBaseAddr(void)
{
    return FMC->DFBA;
}


int32_t FMC_GetBootSource(void)
{
    if (FMC->ISPCTL & FMC_ISPCTL_BS_Msk)
        return 1;
    else
        return 0;
}


/**
 * @brief      read configure 
 * @param[in]  config_idx, config index
 * @param[in]  u32Config, buffer for configure, it should be 16 byte aligen, but only first 4 byte is configure
 * @return     
 * @history  - V1.0, 2019-3-20, first implementation.
 */
int32_t FMC_ReadConfig(uint32_t config_idx, uint32_t *u32Config)
{

	FMC_Read(FMC_CONFIG_BASE + config_idx * 4, u32Config);
    return 0;
}


/**
  * @brief    Write User Configuration
  * @param[in]    u32Config  The word array to store data.
  * @param[in]    u32Count   Maximum length of "u32Config".
  * @retval   0   Success
  * @retval   -1  Failed
  */
int32_t FMC_WriteConfig(uint32_t *u32Config, uint32_t u32Count)
{
	uint32_t i;
	
    if (u32Count < 1)
        return -1;

    FMC_ENABLE_CFG_UPDATE();
    FMC_Page_Erase(FMC_CONFIG_BASE);

	for(i=0;i<u32Count;i++){
		FMC_Write(FMC_CONFIG_BASE+i*4, &u32Config[i]);
	}


    FMC_DISABLE_CFG_UPDATE();
    return 0;
}


/**
  * @brief    Calculate and read the CRC32 checksum of a specified flash area.
  * @param[in]    addr     Start address of the flash area to be executed CRC32 checksum calculation.
  * @param[in]    count    Number of bytes to be calculated.
  * @param[out]   chksum   If success, it will contain the result of CRC32 checksum calculation.
  * @retval   0   Success
  * @retval   -1  Invalid parameter.
  */
int32_t FMC_GetCRC32Sum(uint32_t addr, uint32_t count, uint32_t *chksum)
{
    FMC->ISPCMD = FMC_ISPCMD_CAL_CRC32;
    FMC->ISPADDR = addr;
    FMC->ISPDAT  = count;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;


    FMC->ISPCMD = FMC_ISPCMD_READ_CRC32;
    FMC->ISPADDR = addr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    *chksum = FMC->ISPDAT;

    return 0;
}

/**
  * @brief    FMC uninit
  * @param[in]    u32Config  The word array to store data.
  * @param[in]    u32Count   Maximum length of "u32Config".
  * @retval   0   Success
  * @retval   -1  Failed
  */
int FMC_UnInit(void) 
{
    FMC_DISABLE_AP_UPDATE();
    FMC_DISABLE_LD_UPDATE();
    FMC_DISABLE_ISP();         
    return 0;
}

/**
 * @brief      Read INFO DATA 
 * @param[in]  none
 * @return     none
 * @history  - V1.0, 2019-3-20, first implementation.
 */
uint32_t flash_ReadInfo(uint32_t u32Addr)
{
//    SYS_UnlockReg();
//    FMC_Open();
	SYS_UnlockReg();
    FMC_Open();
    FMC_ENABLE_SP_UPDATE();
    FMC_ENABLE_AP_UPDATE();
    FMC->ISPCMD = 0x0B;
    FMC->ISPADDR = u32Addr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;

    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    return FMC->ISPDAT;
}



