/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file lowpower.c
 * @brief 
 *
 * @history - V0.01, 2019-03-20, yangyaming, first implementation.
*******************************************************************************/
#include "lowpower.h"

/**
* @brief  This function request to enter sleep
* @param[in]  anac: where anac is anac module base address
* @return   none
*/
void LP_RequestSleep(ANAC_T *anac)
{
	anac->LP_FL_CTRL0 |= SLEEP_REQUEST_Msk;
}
/**
* @brief  This function set sleep time
* @param[in]  anac: where anac is anac module base address
* @param[in]  u32ClkCnt: where u32ClkCnt is 32k clock cnt num
* @return   none
*/
void LP_SetSleepTime(ANAC_T *anac,uint32_t u32ClkCnt)
{
	anac->LP_SLT = u32ClkCnt;
}

 /**
* @brief  This function disable external wake up function
* @param[in]  anac: where anac is anac module base address
* @param[in]  u8ExtWkDis: where u8ExtWkDis determine whether to wake up by GPIO
*																									0:gpio can wake up
*																									1:gpio can notwake up
* @return   none
*/
void LP_SetExternalWake(ANAC_T *anac,uint8_t u8ExtWkDis)
{	
	if(u8ExtWkDis)
		anac->LP_FL_CTRL0 |= EXT_WKUP_DISABLE_Msk;
	else
		anac->LP_FL_CTRL0 &= ~EXT_WKUP_DISABLE_Msk;
}

/**
* @brief  This function wait to clear wake up flag
* @param[in]  anac: where anac is anac module base address
* @param[in]  u32Mask: bit need to be cleared
*											LP_CLOCK_GATE_INT_BIT_Msk
*											LP_SLEEP_MODE_INT_BIT_Msk
* @return   none
*/
void LP_ClearWakeFlag(ANAC_T *anac,uint32_t u32Mask)
{
	anac->LP_INT_CTRL |= u32Mask;
}

/**
* @brief  This function used to set digital reset time
* @param[in]  anac: where anac is anac module base address
* @param[in]  u8Clk32Cnt: where u8Clk32Cnt is 32k clock period cnt
* @return   none
*/
void LP_SetDigitalDelay(ANAC_T *anac,uint8_t u8Clk32Cnt)
{
	anac->LP_1P5V_RST &= ~(RESET_DLY_TIME_Msk);
	anac->LP_1P5V_RST |= (u8Clk32Cnt << RESET_DLY_TIME_Pos);
	anac->LP_1P5V_RST &= ~RESET_EN_Msk;
}

/**
* @brief  This function used to disable digital reset
* @param[in]  anac: where anac is anac module base address
* @return   none
*/
void LP_DisableDigitalRst(ANAC_T *anac)
{
	anac->LP_1P5V_RST &= ~(RESET_EN_Msk);
}


 /**
* @brief  This function Enable low power LDO
* @param[in]  anac: where anac is anac module base address
* @return   none
*/
void LP_EnableLpLdo(ANAC_T *anac)
{	
	anac->LP_FL_CTRL1 |= EN_DVDD_LP_Msk;
}


 /**
* @brief  This function set deep sleep mode config
* @param[in]  anac: where anac is anac module base address
* @param[in]  wkup_mode: where wkup_mode is wake up mode select
*								LP_WKUP_MODE_SEL_GPIO
*								LP_WKUP_MODE_SEL_32K 
* @return   none
*/
void LP_SetDeepSleepConfig(ANAC_T *anac)
{
    LP_InterruptEnable(ANAC,SLEEP_INT_EN_Msk);  //DeepSleep/Sleep wake up interrupt enable
    anac->LP_MISC &= ~RC16M_CTRL_Msk;           //16MĶ	        
    anac->LP_FL_CTRL0 |= 	PM_SLEEP_EN_Msk    	//BandGap
                            | DVDD_HP_SLEEP_EN_Msk  	//hp LDO
                            | LDO_DPLL_SLEEP_EN_Msk  // ldo_dpll
                            | DPLL_SLEEP_EN_Msk     	//DPLL
                            | CE_INT_SLEEP_EN_Msk    //XTAL buffer
                            | PWR_UP_SLEEP_EN_Msk    //XTAL
                            | RCO16M_SLEEP_EN_Msk		//16MRC
                            | SLEEP_EN_Msk;
//    if(LP_WKUP_MODE_SEL_GPIO == u8wkupMode)
//        anac->LP_FL_CTRL0 |= RCO32K_SLEEP_EN_Msk;
}

 /**
* @brief  This function set standby mode config
* @param[in]  anac: where anac is anac module base address
* @param[in]  wkup_mode: where wkup_mode is wake up mode select
*								LP_WKUP_MODE_SEL_GPIO
*								LP_WKUP_MODE_SEL_32K 
* @param[in]  u8SramRetention: where u8SramRetention is the flag determines 
*             whether the data in sram is saved. 
*								LP_WKUP_SRAM_DATA_SAVED
*								LP_WKUP_SRAM_DATA_LOSTED
* @return   none
*/
void LP_SetStandbyConfig(ANAC_T *anac,uint8_t u8SramRetention)
{	
    LP_InterruptEnable(anac,STANDBY_INT_EN_Msk);
    anac->LP_FL_CTRL0 |= SLEEP_EN_Msk | PM_SLEEP_EN_Msk | DVDD_HP_SLEEP_EN_Msk;
    
    anac->LP_FL_CTRL1 &= ~(PWR_UP_Msk | CE_INT_Msk | EN_DVDD_LP_Msk 
                           | EN_DPLL_Msk | EN_LDO_DPLL_Msk);
    if(LP_WKUP_SRAM_DATA_SAVED == u8SramRetention){
		anac->LP_FL_CTRL1 |= EN_DVDD_LP_Msk;
		anac->LP_FL_CTRL1 &= ~EN_DVDD_RAM_Msk;
    }
    //close 
    anac->LP_ANA_CTRL0 &= ~EN_LDO_Msk;		
}
